// UnzipFile.cpp: implementation of the CUnzipFile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "NPUnzipFile.h"
#include "NPZipData.h"
#include "ATLPath.h"
#define SIZECENTRALDIRITEM (0x2e)
#define SIZEZIPLOCALHEADER (0x1e)


#ifndef UNZ_BUFSIZE
#define UNZ_BUFSIZE (16384)
#endif

#ifndef UNZ_MAXFILENAMEINZIP
#define UNZ_MAXFILENAMEINZIP (512)
#endif

#define BUFREADCOMMENT (0x400)


UnzipInfo::UnzipInfo()
{
	m_pCurrentFileInfoInZip		= NULL;
}

void UnzipInfo::Alloc_CurrentFileInfoInZip()
{
	if (!m_pCurrentFileInfoInZip)
	{	
		m_pCurrentFileInfoInZip = new ReadFileInfoInZip;
	}
}

void UnzipInfo::Free_CurrentFileInfoInZip()
{
	if (m_pCurrentFileInfoInZip)
	{
		delete m_pCurrentFileInfoInZip;
		m_pCurrentFileInfoInZip = NULL;
	}	
}

UnzipInfo::~UnzipInfo()
{
	Free_CurrentFileInfoInZip();
}

ReadFileInfoInZip::ReadFileInfoInZip()
{
	m_pReadBuffer			= new BYTE[UNZ_BUFSIZE];
	m_lStreamFlag			= 0;
	m_lPosExtraField		= 0;
	m_lCrc32				= 0;
	m_Stream.total_out		= 0;
}


ReadFileInfoInZip::~ReadFileInfoInZip()
{
	delete[] m_pReadBuffer;
}







//////////////////////////////////////////////////////////////////////
// DirectoryInfo
//////////////////////////////////////////////////////////////////////
DirectoryInfo::DirectoryInfo()
{
	memset(m_szDirectory, 0, MAX_DIRECTORY);
	m_vChildDirectory.clear();
	m_pParentDirectory	= NULL;
}
DirectoryInfo::~DirectoryInfo()
{
	m_pParentDirectory	= NULL;
	m_vChildDirectory.clear();
}
DirectoryInfo * DirectoryInfo::GetParentDiretory()
{
	return m_pParentDirectory;
}
void DirectoryInfo::SetParentDiretory(DirectoryInfo * pParentDirectory)
{
	m_pParentDirectory	= pParentDirectory;
}
DirectoryInfo * DirectoryInfo::GetChildDirectory(int nIndex)
{
	int nMax	= GetChildDirectoryCount();
	if(nIndex >= 0 && nIndex < nMax)
	{
		DirectoryInfo * stDirectoryInfo = &m_vChildDirectory[nIndex];
		return stDirectoryInfo;
	}
	
	return NULL;
}

int DirectoryInfo::GetChildDirectoryCount()
{
	return m_vChildDirectory.size();
}

BOOL DirectoryInfo::InsertChildDirectory(LPCSTR pszcDirctory)
{
	DirectoryInfo stDirectoryInfo;
	strncpy(stDirectoryInfo.m_szDirectory, pszcDirctory, MAX_DIRECTORY);
	m_vChildDirectory.push_back(stDirectoryInfo);
	return TRUE;
}





//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CNPUnzipFile::CNPUnzipFile()
{
	m_pZipData					= NULL;
	m_bDetectZlibMemoryLeaks	= TRUE;

}

void CNPUnzipFile::Close()
{
	if (uf.m_pCurrentFileInfoInZip)
        CloseCurrentFile();	
	
	if (m_list.GetCount())
	{
		POSITION pos = m_list.GetHeadPosition();
		while (pos)
			delete[] m_list.GetNext(pos);		
	}
	
	m_pZipData				= NULL;
}


int CNPUnzipFile::CloseCurrentFile()
{

	if (uf.m_pCurrentFileInfoInZip == NULL)
		return UNZIPFILE_ERROR_CHECK_HEADER; 

	if (uf.m_pCurrentFileInfoInZip->m_lRestReadUncompressed == 0)
	{
		if (uf.m_pCurrentFileInfoInZip->m_lCrc32 != uf.m_pCurrentFileInfoInZip->m_lCrc32_wait)
		{			
			return UNZIPFILE_ERROR_CRC;
		}
	}

	
	if (uf.m_pCurrentFileInfoInZip->m_lStreamFlag)
		deflateSetDictionary(NULL, NULL, NULL);

	uf.Free_CurrentFileInfoInZip();
	
	return UNZIPFILE_SUCCESS;
}


void CNPUnzipFile::GoToFirstFile()
{
	if(m_pZipData == NULL) return ;
	
	uf.m_lPosCentralDir = uf.m_lCentralDirOffset;
	uf.m_lFileNum = 0;
	GetCurrentFileInfoInternal(
									uf.m_FileInfo,
									uf.m_FileInfoInternal,
									NULL,
									0,
									NULL,
									0,
									NULL,
									0
							 );
	uf.m_lFileOK = TRUE;
}


int CNPUnzipFile::FindFileInZip(LPSTR szFileName, BOOL bCaseSensitive)
{

	if(m_pZipData == NULL) return UNZIPFILE_ERROR_ZIPDATA;
	
	if (strlen(szFileName) >= UNZ_MAXFILENAMEINZIP)
       return UNZIPFILE_ERROR_FILENAME_SIZE;
	
	if (!uf.m_lFileOK)
		return UNZIPFILE_ERROR_CURRENTFILE;
	
	uLong lSavedFileNum		= uf.m_lFileNum;
	uLong lPosCentralDir	= uf.m_lPosCentralDir;
	
	GoToFirstFile();
	
	BOOL bNextFile = TRUE;
	while (bNextFile)
	{
		char szCurrentFileName[UNZ_MAXFILENAMEINZIP+1];
		GetCurrentFileInfo(NULL, szCurrentFileName, UNZ_MAXFILENAMEINZIP, NULL, 0, NULL, 0);
		if (StringFileNameCompare(szCurrentFileName, szFileName, bCaseSensitive)==0)
			return UNZIPFILE_SUCCESS;
		
		bNextFile = GoToNextFile();
	}
	
	uf.m_lFileNum		= lSavedFileNum ;
	uf.m_lPosCentralDir = lPosCentralDir ;
	
	return UNZIPFILE_ERROR;
}



BOOL CNPUnzipFile::GoToNextFile()
{
	if(m_pZipData == NULL) return FALSE;
	
	if (!uf.m_lFileOK)
		return FALSE;

	if (uf.m_lFileNum + 1 == uf.gi.m_lNumberEntry)
		return FALSE;

	uf.m_lPosCentralDir		+=	SIZECENTRALDIRITEM + 
								uf.m_FileInfo.m_lFilenameSize +
								uf.m_FileInfo.m_lFileExtraSize + 
								uf.m_FileInfo.m_lFileCommentSize;
	
	uf.m_lFileNum++;
	GetCurrentFileInfoInternal(uf.m_FileInfo, uf.m_FileInfoInternal, NULL, 0, NULL, 0, NULL, 0);
	uf.m_lFileOK = TRUE;
	return TRUE;
}

int CNPUnzipFile::CheckCurrentFileCoherencyHeader(
													uInt & nSizeVar,
													uLong & lExtraOffset,
													uLong & lExtraSize
												)
{
	if(m_pZipData == NULL) return UNZIPFILE_ERROR_ZIPDATA;
	
	nSizeVar		= 0;
	lExtraOffset	= 0;
	lExtraSize		= 0;
	
	m_pZipData->Seek(uf.m_FileInfoInternal.m_lCurrentFileOffset + uf.m_lUnzipFileSize);
	
	uLong lCheck;
	GetLong(lCheck);
	if (lCheck != 0x04034b50)
	{
		return UNZIPFILE_ERROR_CHECK_HEADER;
	}
	
	uLong lData;
	GetShort(lData);
	
	uLong lFlags;
	GetShort(lFlags);
	
	GetShort(lData);
	if (lData != uf.m_FileInfo.m_lCompressionMethod)
	{
		return UNZIPFILE_ERROR_COMPRESS_METHOD;
	}
	
    if ((uf.m_FileInfo.m_lCompressionMethod != 0) && (uf.m_FileInfo.m_lCompressionMethod != Z_DEFLATED))
	{
		return UNZIPFILE_ERROR_COMPRESS_METHOD;
	}
	
	GetLong(lData);
	
	GetLong(lData);
	if ((lData != uf.m_FileInfo.m_lCrc) && ((lFlags & 8)==0))
	{
		return UNZIPFILE_ERROR_CRC;
	}
	
	//   
	GetLong(lData);
	if ((lData != uf.m_FileInfo.m_lCompressedSize) && ((lFlags & 8) == 0))
	{
		return UNZIPFILE_ERROR_COMPRESS_SIZE;
	}
	
	//  Ǯ  
	GetLong(lData);
	if ((lData!=uf.m_FileInfo.m_lUncompressedSize) && ((lFlags & 8)==0))
	{
		return UNZIPFILE_ERROR_UNCOMPRESS_SIZE;
	}
	
	// ϸ   
	uLong lFileNameSize;
	GetShort(lFileNameSize);
	if ((lFileNameSize != uf.m_FileInfo.m_lFilenameSize))
	{
		return UNZIPFILE_ERROR_FILENAME_SIZE;
	}
	
	nSizeVar = (uInt)lFileNameSize;

	uLong lExtraFieldSize;
	GetShort(lExtraFieldSize);

	lExtraOffset= uf.m_FileInfoInternal.m_lCurrentFileOffset + SIZEZIPLOCALHEADER + lFileNameSize;
	lExtraSize	= (uInt)lExtraFieldSize;

	nSizeVar	+= (uInt)lExtraFieldSize;
	
	return UNZIPFILE_SUCCESS;
}



int CNPUnzipFile::OpenCurrentFile()
{
	if(m_pZipData == NULL) return UNZIPFILE_ERROR_ZIPDATA;
	
	if (!uf.m_lFileOK)
		return UNZIPFILE_ERROR_CURRENTFILE;
	
	if (!uf.m_lFileOK)
		return UNZIPFILE_ERROR_CURRENTFILE;
	
	//   ݱ
    if (uf.m_pCurrentFileInfoInZip)
        CloseCurrentFile();
	
	uInt	nSizeVar;
	uLong	lExtraOffset; 
	uLong	lExtraSize;  
	
	CheckCurrentFileCoherencyHeader(nSizeVar, lExtraOffset, lExtraSize);
	
	//  ü ޸𸮿 
	uf.Alloc_CurrentFileInfoInZip();
	
	
	uf.m_pCurrentFileInfoInZip->m_lExtraOffset	= lExtraOffset;
	uf.m_pCurrentFileInfoInZip->m_lExtraSize	= lExtraSize;
	
	
	if ((uf.m_FileInfo.m_lCompressionMethod!=0) && (uf.m_FileInfo.m_lCompressionMethod!=Z_DEFLATED))
		return UNZIPFILE_ERROR_COMPRESS_METHOD;
	
	
	uf.m_pCurrentFileInfoInZip->m_lCrc32_wait		= uf.m_FileInfo.m_lCrc;
	uf.m_pCurrentFileInfoInZip->m_lCompressionMethod= uf.m_FileInfo.m_lCompressionMethod;
	
	uf.m_pCurrentFileInfoInZip->m_lUnzipFileSize	=uf.m_lUnzipFileSize;
	
	
	if (uf.m_FileInfo.m_lCompressionMethod != 0)
	{
		uf.m_pCurrentFileInfoInZip->m_Stream.zalloc = (alloc_func)MyAlloc;
		uf.m_pCurrentFileInfoInZip->m_Stream.zfree = (free_func)MyFree;
		uf.m_pCurrentFileInfoInZip->m_Stream.opaque = m_bDetectZlibMemoryLeaks ? &m_list : 0;
	     
		int nError = inflateInit2(&uf.m_pCurrentFileInfoInZip->m_Stream, -MAX_WBITS);
		
		if (nError == Z_OK)
			uf.m_pCurrentFileInfoInZip->m_lStreamFlag=1;		
	}
	
	uf.m_pCurrentFileInfoInZip->m_lRestReadCompressed	= uf.m_FileInfo.m_lCompressedSize ;
	uf.m_pCurrentFileInfoInZip->m_lRestReadUncompressed	= uf.m_FileInfo.m_lUncompressedSize ;

	
	uf.m_pCurrentFileInfoInZip->m_lPosInZipFile =	uf.m_FileInfoInternal.m_lCurrentFileOffset + 
													SIZEZIPLOCALHEADER + 
													nSizeVar;
	
	uf.m_pCurrentFileInfoInZip->m_Stream.avail_in = (uInt)0;
	
	return UNZIPFILE_SUCCESS;
}

int CNPUnzipFile::GetCurrentFileInfo (	UnzipFileInfo* pFileInfo,
										LPSTR	szFileName,
										uLong	lFileNameBufferSize,
										void	*pExtraField,
										uLong	lExtraFieldBufferSize,
										LPSTR	szComment,
										uLong	lCommentBufferSize)
{

	
	UnzipFileInfo temp;
	if (!pFileInfo)
		pFileInfo = &temp;

	UnzipFileInfo_Internal temp1;
	return GetCurrentFileInfoInternal(*pFileInfo, temp1, szFileName, lFileNameBufferSize,
				pExtraField, lExtraFieldBufferSize, szComment, lCommentBufferSize);
}

BOOL CNPUnzipFile::GetDirectoryInfo(DirectoryInfo * pDirectoryInfo)
{
	if(m_pZipData == NULL || pDirectoryInfo == NULL) return FALSE;
	
	char szPreDirectoryName[MAX_DIRECTORY];
	memset(szPreDirectoryName, 0, MAX_DIRECTORY);
	sprintf(pDirectoryInfo->m_szDirectory, "root");
	GoToFirstFile();
	do
	{
		UnzipFileInfo fileInfo;
		DirectoryInfo stDirectoryInfo;
		int nResult = GetCurrentFileInfo(&fileInfo, (LPSTR)stDirectoryInfo.m_szDirectory, MAX_DIRECTORY);
		if(nResult == UNZIPFILE_SUCCESS) 
		{
			if(fileInfo.m_lExternal_FA == 16)
			{
				//丮 
				InsertDirectory(pDirectoryInfo, stDirectoryInfo.m_szDirectory);
				memset(szPreDirectoryName, 0, MAX_DIRECTORY);				
			}
			else	
			{
				// 
				char * pszFileName	= ::PathFindFileNameA(stDirectoryInfo.m_szDirectory);
				int nFileNameLen	= strlen(pszFileName);
				int nDirectoryLen	= strlen(stDirectoryInfo.m_szDirectory);
				char szDirectoryName[MAX_DIRECTORY];
				memset(szDirectoryName, 0, MAX_DIRECTORY);
				strncpy(szDirectoryName, stDirectoryInfo.m_szDirectory, nDirectoryLen-nFileNameLen-1);
				if(stricmp(szDirectoryName, szPreDirectoryName) == 0)
				{
				}
				else
				{
					strcpy(stDirectoryInfo.m_szDirectory, szDirectoryName);
					strcpy(szPreDirectoryName, szDirectoryName);
					InsertDirectory(pDirectoryInfo, stDirectoryInfo.m_szDirectory);
				}				
			}			
		}
	
	}while(GoToNextFile());
	
	
	return TRUE;
}


int CNPUnzipFile::ReadCurrentFile(void* pBuffer, UINT nBufferSize)
{

	if(m_pZipData == NULL) return UNZIPFILE_ERROR_ZIPDATA;
	
	if (!uf.m_pCurrentFileInfoInZip)
	{
		return UNZIPFILE_ERROR_HEADDATA;
	}

	if ((nBufferSize == 0) || !pBuffer)
		return UNZIPFILE_ERROR_DATA;

	uf.m_pCurrentFileInfoInZip->m_Stream.next_out		= (BYTE*)pBuffer;
	uf.m_pCurrentFileInfoInZip->m_Stream.avail_out	= (uInt)nBufferSize;	
	if (nBufferSize > uf.m_pCurrentFileInfoInZip->m_lRestReadUncompressed)
	{
		uf.m_pCurrentFileInfoInZip->m_Stream.avail_out = (uInt)uf.m_pCurrentFileInfoInZip->m_lRestReadUncompressed;
	}
	
	uInt iRead = 0;
	
	while (uf.m_pCurrentFileInfoInZip->m_Stream.avail_out>0)
	{
		if ((uf.m_pCurrentFileInfoInZip->m_Stream.avail_in==0) && (uf.m_pCurrentFileInfoInZip->m_lRestReadCompressed>0))
		{
			uInt uReadThis = UNZ_BUFSIZE;
			if (uf.m_pCurrentFileInfoInZip->m_lRestReadCompressed < uReadThis)
				uReadThis = (uInt)uf.m_pCurrentFileInfoInZip->m_lRestReadCompressed;
			if (uReadThis == 0)
				return 0;
			
			m_pZipData->Seek(
									uf.m_pCurrentFileInfoInZip->m_lPosInZipFile + 
									uf.m_pCurrentFileInfoInZip->m_lUnzipFileSize
							);
			m_pZipData->ReadData(uf.m_pCurrentFileInfoInZip->m_pReadBuffer, uReadThis);
			
			uf.m_pCurrentFileInfoInZip->m_lPosInZipFile			+= uReadThis;
			uf.m_pCurrentFileInfoInZip->m_lRestReadCompressed	-=uReadThis;
			uf.m_pCurrentFileInfoInZip->m_Stream.next_in		= (Bytef*)uf.m_pCurrentFileInfoInZip->m_pReadBuffer;
			uf.m_pCurrentFileInfoInZip->m_Stream.avail_in		= (uInt)uReadThis;
		}

		if (uf.m_pCurrentFileInfoInZip->m_lCompressionMethod == 0)
		{
			uInt lDoCopy;
			if (uf.m_pCurrentFileInfoInZip->m_Stream.avail_out < uf.m_pCurrentFileInfoInZip->m_Stream.avail_in)
				lDoCopy = uf.m_pCurrentFileInfoInZip->m_Stream.avail_out ;
			else
				lDoCopy = uf.m_pCurrentFileInfoInZip->m_Stream.avail_in ;
			
			memcpy(uf.m_pCurrentFileInfoInZip->m_Stream.next_out, uf.m_pCurrentFileInfoInZip->m_Stream.next_in, lDoCopy);
					
			uf.m_pCurrentFileInfoInZip->m_lCrc32 = crc32(
															uf.m_pCurrentFileInfoInZip->m_lCrc32,
															uf.m_pCurrentFileInfoInZip->m_Stream.next_out,
															lDoCopy
														);

			uf.m_pCurrentFileInfoInZip->m_lRestReadUncompressed	-= lDoCopy;
			uf.m_pCurrentFileInfoInZip->m_Stream.avail_in		-= lDoCopy;
			uf.m_pCurrentFileInfoInZip->m_Stream.avail_out		-= lDoCopy;
			uf.m_pCurrentFileInfoInZip->m_Stream.next_out		+= lDoCopy;
			uf.m_pCurrentFileInfoInZip->m_Stream.next_in		+= lDoCopy;
            uf.m_pCurrentFileInfoInZip->m_Stream.total_out		+= lDoCopy;
			iRead += lDoCopy;
		}
		else
		{
			uLong lTotalOutBefore = uf.m_pCurrentFileInfoInZip->m_Stream.total_out;
			const Bytef *bufBefore = uf.m_pCurrentFileInfoInZip->m_Stream.next_out;
			int flush=Z_SYNC_FLUSH;

			int nError = inflate(&uf.m_pCurrentFileInfoInZip->m_Stream,flush);

			uLong lTotalOutAfter	= uf.m_pCurrentFileInfoInZip->m_Stream.total_out;
			uLong lOutThis			= lTotalOutAfter - lTotalOutBefore;
			
			uf.m_pCurrentFileInfoInZip->m_lCrc32 = crc32(uf.m_pCurrentFileInfoInZip->m_lCrc32, bufBefore, (uInt)(lOutThis));

			uf.m_pCurrentFileInfoInZip->m_lRestReadUncompressed -= lOutThis;

			iRead += (uInt)(lTotalOutAfter - lTotalOutBefore);
            
			if (nError==Z_STREAM_END)
			{
				// [:2008-12-19] Memory Leak  : InflateEnd()  -> zip ޸ ϴ Լ
				inflateEnd(&uf.m_pCurrentFileInfoInZip->m_Stream);
				return iRead;
			}
		}
	}

	return iRead;
}

CNPUnzipFile::~CNPUnzipFile()
{

}

void CNPUnzipFile::GetByte(int & nInt)
{
	m_pZipData->GetByte(nInt);    
}

void CNPUnzipFile::GetShort(uLong & lLong)
{
    m_pZipData->GetShort(lLong);
}

void CNPUnzipFile::GetLong(uLong & lLong)
{
	m_pZipData->GetLong(lLong);
}

int CNPUnzipFile::StringFileNameCompare(LPSTR strFileName1, LPSTR strFileName2, BOOL bCaseSensitive)
{
	return bCaseSensitive ? strcmp(strFileName1, strFileName2) : stricmp(strFileName1, strFileName2);
}
DirectoryInfo * CNPUnzipFile::FindDirectory(DirectoryInfo * pDirectoryInfo, LPSTR pszcDirectory)
{
	if(pDirectoryInfo == NULL)	return NULL;
	int nMax = pDirectoryInfo->GetChildDirectoryCount();
	for(int nIndex=0; nIndex<nMax; nIndex++)
	{
		DirectoryInfo * pCurDirectoryInfo = pDirectoryInfo->GetChildDirectory(nIndex);
		if(pCurDirectoryInfo != NULL)
		{
			if(stricmp(pCurDirectoryInfo->m_szDirectory, pszcDirectory) == 0)
			{
				return  pCurDirectoryInfo;
			}
		}
	}
	
	return NULL;
}
void CNPUnzipFile::InsertDirectory(DirectoryInfo * pDirectoryInfo, LPSTR pszcDirectory)
{
	int nLength = strlen(pszcDirectory);
	if(nLength <= 0)	return;
	DirectoryInfo * pParentDirectoryInfo	= pDirectoryInfo;
	char * szDelimiter	= "/";
	char * pToken		= (char*)_mbstok((unsigned char*)pszcDirectory, (unsigned char*)szDelimiter);
	while(pToken != NULL)
	{
		DirectoryInfo * pChildDirectoryInfo = FindDirectory(pParentDirectoryInfo, pToken);
		if(pChildDirectoryInfo == NULL)
		{
			pParentDirectoryInfo->InsertChildDirectory(pToken);
			int nChildCount		= pParentDirectoryInfo->GetChildDirectoryCount();
			pChildDirectoryInfo	= pParentDirectoryInfo->GetChildDirectory(nChildCount-1);
			pChildDirectoryInfo->SetParentDiretory(pParentDirectoryInfo);
			pParentDirectoryInfo= pChildDirectoryInfo;
		}
		else
		{
			pParentDirectoryInfo= pChildDirectoryInfo;
		}
		pToken = (char*)_mbstok(NULL, (unsigned char*)szDelimiter);
	}
	
}
uLong CNPUnzipFile::SearchCentralDir()
{	
	if(m_pZipData == NULL) return (uLong)UNZIPFILE_ERROR_ZIPDATA;
	
	uLong lMaxBack=0xffff; 
	
	uLong lSizeFile = m_pZipData->GetZipDataSize();
	
	if (lMaxBack > lSizeFile)
		lMaxBack = lSizeFile;
	
	BYTE * pBuffer = new BYTE[BUFREADCOMMENT + 4];
	
	uLong lBackRead = 4;
	uLong lPosFound = 0;
	
	while ((lBackRead < lMaxBack) && !lPosFound)
	{
		uLong lReadSize, lReadPos;
		
		if (lBackRead + BUFREADCOMMENT > lMaxBack) 
			lBackRead = lMaxBack;
		else
			lBackRead += BUFREADCOMMENT;

		lReadPos = lSizeFile - lBackRead ;
		
		lReadSize = ((BUFREADCOMMENT + 4) < (lSizeFile - lReadPos)) ? 
						(BUFREADCOMMENT + 4) : (lSizeFile - lReadPos);

		m_pZipData->Seek(lReadPos);
		m_pZipData->ReadData(pBuffer, lReadSize);
		
		for (int i= (int)lReadSize - 3; (i--) > 0 ;)
			if (((*(pBuffer+i)) == 0x50) && ((*(pBuffer+i+1))==0x4b) && 
				((*(pBuffer+i+2))==0x05) && ((*(pBuffer+i+3))==0x06))
			{
				lPosFound = lReadPos + i;
				break;
			}
	
	}
	
	delete[] pBuffer;
	
	return lPosFound;
	
}

void CNPUnzipFile::DosDateToTmuDate(UnzipFileInfo &file_info)
{
	(file_info);
	//CTime t(HIWORD(file_info.m_lDosDate), LOWORD(file_info.m_lDosDate));
	//file_info.m_Tmu_date = t;
}


int CNPUnzipFile::GetCurrentFileInfoInternal( 
												UnzipFileInfo & pFileinfo, 
												UnzipFileInfo_Internal & pFileInfoInternal, 
												LPSTR	szFileName,
												uLong	lFileNameBufferSize, 
												void	*pExtraField, 
												uLong	lExtraFieldBufferSize,
												LPSTR	szComment, 
												uLong	lCommentBufferSize
										   )
{

	if(m_pZipData == NULL) return UNZIPFILE_ERROR_ZIPDATA;
	m_pZipData->Seek(uf.m_lPosCentralDir + uf.m_lUnzipFileSize);
	
	uLong lCheck;
	GetLong(lCheck);
	if (lCheck != 0x02014b50)
	{
		return UNZIPFILE_ERROR;
	}

	GetShort(pFileinfo.m_lVersion);
	GetShort(pFileinfo.m_lversionExtract);
	GetShort(pFileinfo.m_lFlag);
	GetShort(pFileinfo.m_lCompressionMethod);
	GetLong(pFileinfo.m_lDosDate);

    DosDateToTmuDate(pFileinfo);

	GetLong(pFileinfo.m_lCrc);
	GetLong(pFileinfo.m_lCompressedSize);
	GetLong(pFileinfo.m_lUncompressedSize);
	GetShort(pFileinfo.m_lFilenameSize);
	GetShort(pFileinfo.m_lFileExtraSize);
	GetShort(pFileinfo.m_lFileCommentSize);
	GetShort(pFileinfo.m_lDiskNumStart);
	GetShort(pFileinfo.m_lInternal_FA);
	GetLong(pFileinfo.m_lExternal_FA);
	GetLong(pFileInfoInternal.m_lCurrentFileOffset);

	uLong lSeek = pFileinfo.m_lFilenameSize;

	if (szFileName)
	{
		uLong lSizeRead ;
		if (pFileinfo.m_lFilenameSize < lFileNameBufferSize)
		{
			*(szFileName + pFileinfo.m_lFilenameSize) = '\0';
			lSizeRead = pFileinfo.m_lFilenameSize;
		}
		else
			lSizeRead = lFileNameBufferSize;

		if ((pFileinfo.m_lFilenameSize>0) && (lFileNameBufferSize>0))
		{
			m_pZipData->ReadData((BYTE*)szFileName, lSizeRead);			
		}

		lSeek -= lSizeRead;
	}

	
	if (pExtraField)
	{
		uLong lSizeRead ;
		if (pFileinfo.m_lFileExtraSize < lExtraFieldBufferSize)
			lSizeRead = pFileinfo.m_lFileExtraSize;
		else
			lSizeRead = lExtraFieldBufferSize;

		if (lSeek != 0)
		{
			m_pZipData->Seek(lSeek);
			lSeek=0;
		}
		
		if ((pFileinfo.m_lFileExtraSize>0) && (lExtraFieldBufferSize>0))
		{
			m_pZipData->ReadData((BYTE*)pExtraField, lSizeRead);
		}

		lSeek += pFileinfo.m_lFileExtraSize - lSizeRead;
	}
	else
		lSeek+=pFileinfo.m_lFileExtraSize; 

	
	if (szComment)
	{
		uLong lSizeRead ;

		if (pFileinfo.m_lFileCommentSize < lCommentBufferSize)
		{
			*(szComment+pFileinfo.m_lFileCommentSize)='\0';
			lSizeRead = pFileinfo.m_lFileCommentSize;
		}
		else
			lSizeRead = lCommentBufferSize;

		if (lSeek != 0)
		{			
			m_pZipData->Seek(lSeek);
			lSeek=0;
		}

		if ((pFileinfo.m_lFileCommentSize>0) && (lCommentBufferSize>0))
		{
			m_pZipData->ReadData((BYTE*)szComment, lSizeRead);			
		}
			
		lSeek+=pFileinfo.m_lFileCommentSize - lSizeRead;
	}
	else
		lSeek+=pFileinfo.m_lFileCommentSize;

	return UNZIPFILE_SUCCESS;
}


void CNPUnzipFile::Open(INPZipData * pZipData)
{
	
	if(pZipData == NULL) return ;
	
	
	m_pZipData = pZipData;
	
	m_pZipData			= pZipData;
	uLong lPosCentral	= SearchCentralDir();
	
	m_pZipData->Seek(lPosCentral);
	

	uLong uL;
	GetLong(uL);
	
	uLong lNumberDisk;       
	GetShort(lNumberDisk);

	uLong lNumberDiskWithCD;
	GetShort(lNumberDiskWithCD);

	
	GetShort(uf.gi.m_lNumberEntry);


	uLong lNumberEntryCD; 
	GetShort(lNumberEntryCD);
	
	
	if ( (lNumberEntryCD != uf.gi.m_lNumberEntry) || (lNumberDiskWithCD != 0) || (lNumberDisk != 0))
	{
		return ;
	}

	GetLong(uf.m_lCentralDirSize);

	GetLong(uf.m_lCentralDirOffset);
	
	GetShort(uf.gi.m_lCommentSize);

	if ( lPosCentral < uf.m_lCentralDirOffset + uf.m_lCentralDirSize)
	{
		return ;
	}


	uf.m_lUnzipFileSize = lPosCentral - (uf.m_lCentralDirOffset + uf.m_lCentralDirSize);

	uf.m_lPosCentral	= lPosCentral;
	
	GoToFirstFile();
}

void* CNPUnzipFile::MyAlloc(void* opaque, UINT items, UINT size)
{
	void* p = new char[size * items];
	if (opaque)
	{
		CAtlList<void *>* list = (CAtlList<void *> *) opaque;
		list->AddTail(p);
	}
	return p;
}

void CNPUnzipFile::MyFree(void* opaque, void* address)
{
	if (opaque)
	{
		CAtlList<void *>* list = (CAtlList<void *> *) opaque;
		POSITION pos = list->Find(address);
		if (pos)
			list->RemoveAt(pos);
	}
	delete[] address;
}
